<?php

namespace Civi\Certificates;

use Civi\Api4\MessageTemplate;
use Civi\Certificates\Event\AuthorizeEvent;

class Activity {

  /**
   * @return void
   * @throws \CRM_Core_Exception
   * @throws \Civi\API\Exception\UnauthorizedException
   */
  public static function downloadViaURL() {
    $contactID = \CRM_Utils_Request::retrieveValue('cid', 'Positive', NULL, TRUE);
    $activityID = \CRM_Utils_Request::retrieveValue('aid', 'Positive', NULL, TRUE);

    try {
      Activity::downloadPDF($contactID, $activityID);
    }
    catch (\Exception $e) {
      \Civi::log()->error('certificates: ' . $e->getMessage());
      throw $e;
    }
  }

  /**
   * @param int $contactID
   * @param int $activityID
   *
   * @return void
   * @throws \CRM_Core_Exception
   * @throws \Civi\API\Exception\UnauthorizedException
   */
  public static function downloadPDF(int $contactID, int $activityID) {
    $authAll = \CRM_Core_Permission::check('download activity certificates');
    $authOwn = \CRM_Core_Permission::check('download own certificate');

    if (!$authAll && !$authOwn) {
      throw new \CRM_Core_Exception('You are not authorized to download certificates. Ask your administrator for the right permission');
    }

    // Check permissions for contacts downloading their own activity certificate
    if ($authOwn && !$authAll) {
      if (\CRM_Core_Session::getLoggedInContactID() !== $contactID) {
        $event = new AuthorizeEvent('activity', $contactID, $activityID);
        \Civi::dispatcher()->dispatch('civi.certificates.authorize', $event);
        if (!$event->isAuthorized()) {
          throw new \CRM_Core_Exception('You cannot download a certificate for someone else.');
        }
      }
    }

    $activity = \Civi\Api4\Activity::get(FALSE)
      ->addSelect('*', 'custom.*', 'activity_type_id:label')
      ->addWhere('id', '=', $activityID)
      ->execute()
      ->first();

    // Get activity certificate configuration
    $activityTypes = [];
    $configs = preg_split("/\r\n|\n|\r/", \Civi::settings()->get('certificates_activitytypes'));
    foreach ($configs as $config) {
      list($activityTypeID, $messageTemplateID) = explode(':', $config);
      if (empty($messageTemplateID)) {
        \Civi::log()->error('certificates_activitytypes is not configured correctly (missing messageTemplateID).');
        continue;
      }
      $activityTypes[$activityTypeID] = $messageTemplateID;
    }

    if (empty($activityTypes)) {
      throw new \CRM_Core_Exception('No activity types are configured as certificates');
    }

    $context = ['contactId' => $contactID, 'activityId' => $activityID];
    $messageTemplateID = $activityTypes[$activity['activity_type_id']];
    $pdfFormatID = MessageTemplate::get(FALSE)
      ->addSelect('pdf_format_id')
      ->addWhere('id', '=', $messageTemplateID)
      ->execute()
      ->first()['pdf_format_id'];
    $pdfFormatID = (empty($pdfFormatID)) ? 0 : (int) $pdfFormatID;
    if (empty($activityTypes[$activity['activity_type_id']])) {
      throw new \CRM_Core_Exception('Activity type is not configured for certificate');
    }

    $downloadPDF = new PDFCertificate();
    $filename = $activity['activity_type_id:label'] . '_certificate_' . md5($activity['id']);
    $downloadPDF->download($messageTemplateID, $context, $filename, $pdfFormatID);
  }

}
