<?php

namespace Civi\Certificates;

use Civi\Api4\MessageTemplate;
use Civi\Api4\Participant;
use Civi\Certificates\Event\AuthorizeEvent;

class Attendance {

  /**
   * @return void
   * @throws \CRM_Core_Exception
   * @throws \Civi\API\Exception\UnauthorizedException
   */
  public static function downloadViaURL() {
    $contactID = \CRM_Utils_Request::retrieveValue('cid', 'Positive', NULL, TRUE);
    $participantID = \CRM_Utils_Request::retrieveValue('pid', 'Positive', NULL, TRUE);

    try {
      Attendance::downloadPDF($contactID, $participantID);
    }
    catch (\Exception $e) {
      \Civi::log()->error('certificates: ' . $e->getMessage());
      throw $e;
    }
  }

  /**
   * @param int $contactID
   * @param int $participantID
   *
   * @return void
   * @throws \CRM_Core_Exception
   * @throws \Civi\API\Exception\UnauthorizedException
   */
  public static function downloadPDF(int $contactID, int $participantID) {
    $authAll = \CRM_Core_Permission::check('download participant certificates');
    $authOwn = \CRM_Core_Permission::check('download own certificate');

    if (!$authAll && !$authOwn) {
      throw new \CRM_Core_Exception('You are not authorized to download certificates. Ask your administrator for the right permission');
    }

    // Check permissions for participants downloading their own certificate
    if ($authOwn && !$authAll) {
      if (\CRM_Core_Session::getLoggedInContactID() !== $contactID) {
        $event = new AuthorizeEvent('participant', $contactID, $participantID);
        \Civi::dispatcher()->dispatch('civi.certificates.authorize', $event);
        if (!$event->isAuthorized()) {
          throw new \CRM_Core_Exception('You cannot download a certificate for someone else.');
        }
      }
    }

    $participant = Participant::get(FALSE)
      ->addSelect('*', 'status_id:name', 'contact_id.display_name', 'event_id.id', 'event_id.title', 'event_id.Event_Certificate.download_certificate')
      ->addWhere('id', '=', $participantID)
      ->execute()
      ->first();

    if (empty($participant['event_id.Event_Certificate.download_certificate'])) {
      $message = 'Event ID: ' . $participant['event_id.id'] . ' does not have "Download Certificate" enabled';
      \Civi::log()->error($message);
      throw new \CRM_Core_Exception($message);
    }

    if ($participant['status_id:name'] !== 'Attended') {
      throw new \CRM_Core_Exception('You cannot download a certificate for an event you did not attend.');
    }

    $context = ['contactId' => $contactID, 'participantId' => $participantID, 'eventId' => $participant['event_id.id']];
    $messageTemplateID = \Civi::settings()->get('certificates_event_defaultmessagetemplateid');
    $pdfFormatID = MessageTemplate::get(FALSE)
      ->addSelect('pdf_format_id')
      ->addWhere('id', '=', $messageTemplateID)
      ->execute()
      ->first()['pdf_format_id'];
    $pdfFormatID = (empty($pdfFormatID)) ? 0 : (int) $pdfFormatID;

    if (empty($messageTemplateID)) {
      \Civi::log()->error('Event attendance certificate has no message template ID set!');
      throw new \CRM_Core_Exception('Attendance certificate generation disabled.');
    }

    $downloadPDF = new PDFCertificate();
    $filename = $participant['contact_id.display_name'] . '_' . $participant['event_id'] . '_attendance_certificate_' . $participant['event_id.title'];
    $downloadPDF->download($messageTemplateID, $context, $filename, $pdfFormatID);
  }

}
