<?php

namespace Civi\Certificates;

use Civi\Api4\MessageTemplate;
use Civi\Certificates\Event\AuthorizeEvent;

class Membership {

  /**
   * @return void
   * @throws \CRM_Core_Exception
   * @throws \Civi\API\Exception\UnauthorizedException
   */
  public static function downloadViaURL() {
    $contactID = \CRM_Utils_Request::retrieveValue('cid', 'Positive', NULL, TRUE);
    $membershipID = \CRM_Utils_Request::retrieveValue('mid', 'Positive', NULL, TRUE);

    try {
      Membership::downloadPDF($contactID, $membershipID);
    }
    catch (\Exception $e) {
      \Civi::log()->error('certificates: ' . $e->getMessage());
      throw $e;
    }
  }

  /**
   * @param int $contactID
   * @param int $membershipID
   *
   * @return void
   * @throws \CRM_Core_Exception
   * @throws \Civi\API\Exception\UnauthorizedException
   */
  public static function downloadPDF(int $contactID, int $membershipID) {
    $authAll = \CRM_Core_Permission::check('download membership certificates');
    $authOwn = \CRM_Core_Permission::check('download own certificate');

    if (!$authAll && !$authOwn) {
      throw new \CRM_Core_Exception('You are not authorized to download certificates. Ask your administrator for the right permission');
    }

    // Check permissions for participants downloading their own certificate
    if ($authOwn && !$authAll) {
      if (\CRM_Core_Session::getLoggedInContactID() !== $contactID) {
        $event = new AuthorizeEvent('membership', $contactID, $membershipID);
        \Civi::dispatcher()->dispatch('civi.certificates.authorize', $event);
        if (!$event->isAuthorized()) {
          throw new \CRM_Core_Exception('You cannot download a certificate for someone else.');
        }
      }
    }

    $membership = \Civi\Api4\Membership::get(FALSE)
      ->addSelect('id', 'contact_id.display_name')
      ->addWhere('id', '=', $membershipID)
      ->addWhere('status_id.is_current_member', '=', TRUE)
      // For now, we'll retrieve the membership that ends last.
      ->addOrderBy('end_date', 'DESC')
      ->execute()
      ->first();

    if (empty($membership)) {
      throw new \CRM_Core_Exception('You cannot download a certificate if there is no active membership.');
    }

    $context = ['contactId' => $contactID, 'membershipId' => $membershipID];
    $messageTemplateID = \Civi::settings()->get('certificates_membership_defaultmessagetemplateid');
    $pdfFormatID = MessageTemplate::get(FALSE)
      ->addSelect('pdf_format_id')
      ->addWhere('id', '=', $messageTemplateID)
      ->execute()
      ->first()['pdf_format_id'];
    $pdfFormatID = (empty($pdfFormatID)) ? 0 : (int) $pdfFormatID;

    if (empty($messageTemplateID)) {
      \Civi::log()->error('Membership certificate has no message template ID set!');
      throw new \CRM_Core_Exception('Membership certificate generation disabled.');
    }

    $downloadPDF = new PDFCertificate();
    $filename = $membership['contact_id.display_name'] . '_' . $membership['_id'] . '_membership_certificate';
    $downloadPDF->download($messageTemplateID, $context, $filename, $pdfFormatID);
  }

}
